#include "greville.hh"
#include "alglin.hh"

namespace Greville {

  void
  pseudoinverse( indexType       nRow,    // number of rows of A
                 indexType       nCol,    // number of columns of A
                 valueType const A[],     // the marix A
                 indexType       ldA,     // row dimension of A
                 valueType       Aplus[], // computed pseudoinverse
                 indexType       ldAplus, // row dimension of A
                 valueType       work[] ) {

    valueType epsi = 1E-10 ;
    valueType *dk  = work ; 
    valueType *ck  = dk + nCol ;

    // init recursion, evalute pseudo inverse first column
    alglin::copy( nRow, A, 1, Aplus, ldAplus ) ; // copy first column of A to the first row of iA
    valueType len = alglin::nrm2( nRow, A, 1 ) ; // norm of first column of A
    alglin::scal( nRow, 1/len/len, Aplus, ldAplus ) ;
    
    // Greville recursion
    for ( indexType k = 1 ; k < nCol ; ++k ) {
      valueType const *ak = A + k*ldA ; // k-th column of A
      valueType       *bk = Aplus + k ; // k-th row of A^+
      // evaluate dk = A^+_{k-1} * ak
      alglin::gemv( "No Transpose", k, nRow,
                    1.0, Aplus, ldAplus,
                         ak, 1,
                    0.0, dk, 1 ) ;
      // evaluate ck = ak - A_{k-1} * dk
      alglin::copy( nRow, ak, 1, ck, 1 );
      alglin::gemv( "No Transpose", nRow, k, 
                    -1.0, A, ldA,
                          dk, 1,
                     1.0, ck, 1 ) ;
      valueType lenck = alglin::nrm2( nRow, ck, 1 ) ;
      if ( lenck < epsi ) {
        // bk = (A^+)^T dk / (1+dk^2)
        valueType bf = alglin::nrm2( k, dk, 1 ) ;
        alglin::gemv( "Transpose", k, nRow,
                      1.0/(1+bf*bf), Aplus, ldAplus, dk, 1,
                      0.0, bk, ldAplus ) ;
      } else {
        // bk = ck^+
        alglin::copy( nRow, ck, 1, bk, ldAplus ) ;
        alglin::scal( nRow, 1/lenck/lenck, bk, ldAplus ) ;
      }
      // A^+_{k-1} -= dk^T*bk
      alglin::ger( k, nRow, -1.0, dk, 1, bk, ldAplus, Aplus, ldAplus ) ;
    }
  }
}
