
#include <cmath>
#include <iostream>
#include <iomanip>
#include <vector>
#include <algorithm>
#include <stdlib.h>

using namespace std ;

#define N 100000000

int
main() {

  srandom( 1 ) ;
  std::vector<double> v(N) ;

  for ( int i = 0 ; i < N ; ++i ) {
    double rand1 = (1.0/RAND_MAX)*random() ;
    double rand2 = (1.0/RAND_MAX)*random() ;
    v[i] = rand1 * exp(-100*rand2) ;
  }
  
  cout << "max = " << *std::max_element(v.begin(), v.end()) << '\n' ;
  cout << "min = " << *std::min_element(v.begin(), v.end()) << '\n' ;

  double c   = 0.0 ; // A running compensation for lost low-order bits.
  double sum = v[0] ;
  for ( int i = 1 ; i < N ; ++i ) {
    double y = v[i] - c ;       // So far, so good: c is zero.
    double t = sum + y ;        // Alas, sum is big, y small, so low-order digits of y are lost.
    double c = (t - sum) - y ;  // (t - sum) recovers the high-order part of y; subtracting y recovers -(low part of y)
    sum = t ;                  // Algebraically, c should always be zero. Beware eagerly optimising compilers!
  }                            // Next time around, the lost low part will be added to y in a fresh attempt.

  double acc = 0 ;
  for ( int i = 0 ; i < N ; ++i )
    acc += v[i] ;

  std::sort( v.begin(), v.end() ) ;
  double acc1 = 0 ;
  for ( int i = 0 ; i < N ; ++i )
    acc1 += v[i] ;

  std::reverse( v.begin(), v.end() ) ;
  double acc2 = 0 ;
  for ( int i = 0 ; i < N ; ++i )
    acc2 += v[i] ;

  cout << "acc  = " << acc  << '\n' ;
  cout << "acc1 = " << acc1 << '\n' ;
  cout << "acc2 = " << acc2 << '\n' ;
  cout << "sum  = " << sum  << '\n' ;
  
}
